/*
	Copyright (c) 2000, Tinic Uro <tinic@tinic.net> 
	All rights reserved. Please read the LICENSE file 
	which is included in this distribution for details.
*/
#ifndef _ZCONFIG_H_
#define _ZCONFIG_H_
#include "ZSystem.h"

/*
	For maximum speed it is advisable to set this to 1.
	It will draw everything from top to bottom, e.g. each
	new path will be drawn 'under' the current content in
	the bitmap. In case of complex fillings like gradients
	and pattern this can mean a huge speed increase if
	these paths are partially covered by other paths.

	For compability it is though sometimes useful to
	use standard bottom to top rendering, e.g. drawing
	a path will 'cover' the current content of the bitmap.
*/

#define ZRENDERER_TOPBOTTOM				  1 // Top to bottom rendering?

/*
	ZRENDERER_FRAC_Y defines the amount of vertical oversampling.
	The higher the value the slower the drawing. In contrast
	the other values like ZRENDERER_FRAC_X do NOT affect
	drawing speed but rather the precision and limits.

	The sum of
		ZRENDERER_FRAC_X, ZRENDERER_REAL_X and ZRENDERER_REAL_Y
	should be less or equal to 31. The remaining bit is used
	internally.

	ZBUFFERSIZE needs to be set up by judging the possible
	complexity and resolution of the target. To avoid dymamic
	memory allocations during runtime this is a fixed buffer.
	For small displays (f.ex. 320x200) and with low
	vertical oversampling you can reduce this value easily.
	An possible overflow IS handled by the engine, so it won't
	crash at least.

*/

const sint32 ZRENDERER_FRAC_Y   =		  8; // 2^x levels of transparency

const sint32 ZRENDERER_FRAC_X   =		  8; // 2^x levels of transparency
const sint32 ZRENDERER_REAL_X 	=		 12; // real part of x values
const sint32 ZRENDERER_REAL_Y   =	 	 11; // real part of y values
									//  ----
									//   31

const sint32 ZBUFFERSIZE		=   0xFFFFF; // max size of the scanline buffer

/*
	ZGRADIENTSLOTS defines the number of slots gradients 
	will support. Illegal acesses to slots are checked by
	the gradient class. 

	ZGRADIENTBUFFERBITS defines the resolution of the
	gradient. 10 bits are usually enough even for multicolor
	gradients. The lower the value the faster computations
	should be, but you will loose some quality.
*/

const sint32 ZGRADIENTSLOTS		=		  8;
const sint32 ZGRADIENTBUFFERBITS=		 10;


/* 
	ZMIMMAPLEVEL defines the number of mipmap level
	which will be computed for bitmaps. This has
	impact on memory usage. A lower value can mean
	less memory usage but also slower performance
	and uglier display. Depending on your used
	bitmap sizes a value between 6 and 10 is usually
	fine.
*/

const sint32 ZMIMMAPLEVEL		=		 10;

/*
	ZDASHSLOTS defines the maximum allowed slots for 
	dashes.
*/

const sint32 ZDASHSLOTS			=		 16;

#endif
