/*
	Copyright (c) 2000, Tinic Uro <tinic@tinic.net> 
	All rights reserved. Please read the LICENSE file 
	which is included in this distribution for details.
*/
#include "ZDasher.h"
#include "ZLine.h"

ZDasher::ZDasher():
	offset(0) 
{
}

/** Computes dashes for a path.
	Returns result as a new path. Input parameter is a 
	ZDash data structure which should describe the 
	desired dash.
*/
ZPath ZDasher::ComputeDash(const ZPath &from, const ZDash &d)
{
	dash = d;
	outpath.MakeEmpty();
	outshape.MakeEmpty();
	from.Iterate(this);
	curslot = 0;
	curlen = offset;
	for(;curlen>dash.Length(curslot);)
	{
		curlen -= dash.Length(curslot);
		curslot ++;
		if(dash.Length(curslot)==-1)
			curslot = 0;
	}
	return outpath;
}

void ZDasher::NewSegment(ZPoint a, ZPoint b)
{
	if(a!=b)
	{
		ZLine segment(a,b);

		if((curlen+segment.Length()) >= dash.Length(curslot))
		{
			float32 req = dash.Length(curslot) - curlen;
			float32 rea = segment.Length();
			ZPoint split = segment.Split(req/rea);

			if(curslot%2==0)
			{
				outshape.AddLineTo(split);
				curslot ++;
				if(dash.Length(curslot)==-1)
					curslot = 0;
				curlen  = 0;
				NewSegment(split,b);
				return;
			}
			else
			{
				outshape.AddMoveTo(split);
				curslot ++;
				if(dash.Length(curslot)==-1)
					curslot = 0;
				curlen = 0;
				NewSegment(split,b);
				return;
			}
		}
		else
		{
			if(curslot%2==0)
			{
				outshape.AddLineTo(b);
			}
		}
		curlen += segment.Length();
	}
}

void ZDasher::MoveTo(ZPoint point)
{
	outshape.AddMoveTo(point);
	ZSketcher::MoveTo(point);
}

void ZDasher::Close()
{
	ZSketcher::Close();
	outpath += outshape;
	outshape.MakeEmpty();
	curlen  = 0;
	curslot  = 0;
}

void ZDasher::End()
{
	outpath += outshape;
	outshape.MakeEmpty();
}
