/*
	Copyright (c) 2000, Tinic Uro <tinic@tinic.net> 
	All rights reserved. Please read the LICENSE file 
	which is included in this distribution for details.
*/
#include "ZGradient.h"
#include "ZColor.h"

static const sint32	ZGRADIENTBUFFERSIZE	= (1<<ZGRADIENTBUFFERBITS);
static const sint32	ZGRADIENTMAXINTPOS	= (1<<ZGRADIENTBUFFERBITS)-1;

ZGradient::ZGradient():
	mod(MODE_PAD)
{
	{for(sint32 c=0;c<ZGRADIENTSLOTS;c++)
	{
		col[c]= 0;
		pos[c]=-1;
	}}
	{for(sint32 c=0;c<=ZGRADIENTMAXINTPOS;c++)
	{
		pre[c]=0;
	}}
}

/**	Set the color and the position for a certain slot inside the gradient.
	Usually there are 8 slots available, the position should be between
	0 and 1, otherwise it will be ignored or strange behavior can occur.
	To remove a slot entry set the position of the slot to -1.
*/
void ZGradient::SetColor(uint32 color, sint32 slot, float32 position)
{
	if(slot>=0 && slot<ZGRADIENTSLOTS)
	{
		col[slot]=color;
		pos[slot]=position;
		Update();
	}
}

/**	Obtains the color of a certain slot.
*/
uint32 ZGradient::GetColor(sint32 pos) const
{
	switch(mod)
	{
		default:
		case	MODE_PAD:
				return pre[ZMIN(ZGRADIENTMAXINTPOS,ZMAX((sint32)0,pos))];
				break;
		case	MODE_SPREAD:
				return pre[pos&(1<<ZGRADIENTBUFFERBITS) ? 
					ZGRADIENTMAXINTPOS - pos%(1<<ZGRADIENTBUFFERBITS) : 
										 pos%(1<<ZGRADIENTBUFFERBITS) ];
				break;
		case	MODE_REPEAT:
				return pre[pos%(1<<ZGRADIENTBUFFERBITS)];
				break;
	}
}

void ZGradient::Update()
{
	if(pos[0]>=0)
	{
		if(pos[0]>0)
		{
			sint32 end   = (sint32)(pos[0]*(ZGRADIENTMAXINTPOS));
			for(sint32 i=0;i<=end;i++)
			{
				pre[i] = col[0];
			}
		}
		for(sint32 c=0;c<ZGRADIENTSLOTS;c++)
		{
			if(pos[c]>=0)
			{
				if(pos[c+1]>=0)
				{
					sint32 start = (sint32)(pos[c+0]*(ZGRADIENTMAXINTPOS));
					sint32 end   = (sint32)(pos[c+1]*(ZGRADIENTMAXINTPOS));
					sint32 diff  = end-start;
	
					if(diff)
					{
						for(sint32 i=start;i<=end;i++)
						{
							pre[i] = ZColor::Interpolate(col[c+0],col[c+1],(uint08)(((i-start)*255)/diff));
						}
					}
				}
				else
				{
					sint32 start = (sint32)(pos[c+0]*(ZGRADIENTMAXINTPOS));
					for(sint32 i=start;i<=ZGRADIENTMAXINTPOS;i++)
					{
						pre[i] = col[c];
					}
				}
			}
		}
	}
}