/*
	Copyright (c) 2000, Tinic Uro <tinic@tinic.net> 
	All rights reserved. Please read the LICENSE file 
	which is included in this distribution for details.
*/
#ifndef _ZMATRIX_H_
#define _ZMATRIX_H_

class ZMatrix;

#include "ZPoint.h"

/// 2D transformation matrix data structure
class ZMatrix
{
	public:
						 ZMatrix();

				void	 SetRotation(float32 angle);
				void	 SetScaling(ZPoint factor);
				void	 SetTranslation(ZPoint vector);
				void	 SetSkewing(ZPoint angles);
				
				float32	 Rotation() const { return _rotate; }
				ZPoint	 Scaling() const { return _scale; }
				ZPoint	 Translation() const { return _translate; }
				ZPoint	 Skewing() const { return _skewing; }

				ZPoint	 operator*(const ZPoint &from) const;
				ZMatrix	 operator*(const ZMatrix &from) const;
				ZMatrix	 &operator*=(const ZMatrix &from);
				ZMatrix	 &operator=(const ZMatrix &from);
				
				float32	 Determinant();
				void	 Inverse();
				void	 Clear();

	private:

		friend	class	ZPoint;

				void	 Update();
				void	 MakeIdentity();
				void	 SwapRows(ZMatrix &A, sint32 row1, sint32 row2);
				
				float32	 mtx[9];
				float32	_rotate;
				ZPoint	_scale;
				ZPoint	_translate;
				ZPoint	_skewing;
				bool	_isidentity;
};

///
inline ZPoint ZPoint::operator*(const ZMatrix &from) const
{
	if(!from._isidentity) 
	{
		return ZPoint(
			from.mtx[0*3+0]*x + from.mtx[0*3+1]*y + from.mtx[0*3+2],
			from.mtx[1*3+0]*x + from.mtx[1*3+1]*y + from.mtx[1*3+2]
		);
	}
	else
	{
		return *this;
	}
}

///
inline ZPoint &ZPoint::operator*=(const ZMatrix &from)
{
	if(!from._isidentity) 
	{
		Set(
			from.mtx[0*3+0]*x + from.mtx[0*3+1]*y + from.mtx[0*3+2],
			from.mtx[1*3+0]*x + from.mtx[1*3+1]*y + from.mtx[1*3+2]
		);
	}
	return *this;
}


#endif
