/*
	Copyright (c) 2000, Tinic Uro <tinic@tinic.net> 
	All rights reserved. Please read the LICENSE file 
	which is included in this distribution for details.
*/
#include "ZSystem.h"
#include <string>
#include <vector>
#include <list>

using namespace std;

class ZXMLIterator;

class ZXMLParseError
{
	public:
					ZXMLParseError(string e){ error=e; }
	const	string	&Error() const { return error; };

	private:
			string	error;
};

class ZXMLAttribute : public string
{
	public:
									ZXMLAttribute() { };
		const string				&Name() const { return name; };
	private:
			friend class ZXMLElement;
			bool					Parse(iostream &stream, ZXMLIterator *iter=0);
			string					name;
};

class ZXMLElement : public vector<ZXMLElement>
{
	public:
									ZXMLElement() { };
		const string				&Name() const { return name; };
		const vector<ZXMLAttribute>	&Attributes() const { return att; };
//		const vector<string>		&Body() const { return body; }
	private:
			friend class ZXMLDocument;
			bool					Parse(iostream &stream, ZXMLIterator *iter=0);
			string					name;
			vector<ZXMLAttribute>	att;
//			vector<string>			body;
};

class ZXMLDocument
{
	public:
									ZXMLDocument() { };
				void				Parse(string filename, ZXMLIterator *iter=0);
		const	vector<ZXMLElement>	&Root() const { return root; };
	private:
				vector<ZXMLElement>	root;
};

class ZXMLIterator
{
	public:

		virtual	void				ElementNamed(const ZXMLElement &element)=0;
		virtual	void				ElementAttributed(const ZXMLElement &element)=0;
		virtual	void				ElementCompleted(const ZXMLElement &element)=0;
};

