/*
	Copyright (c) 2000, Tinic Uro <tinic@tinic.net>
	All rights reserved. Please read the LICENSE file
	which included in this distribution for details.
*/
#include "ZXML.h"

#ifdef WIN32
#include "ptc.h"
#else
#include <ptc/ptc.h>
#endif

#include "ZRenderer.h"
#include "ZOutliner.h"
#include "ZColor.h"
#include "ZDasher.h"

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <time.h>
#include <string.h>

void convert(uint32 *src, uint32 *dst)
{
	for(int32 c=0;c<((320+160)*240);c+=3)
	{
		sint32 r0=	 ((src[c+0]>>24)&0xFF)
		;
		sint32 g0=	(((src[c+0]>>16)&0xFF)+
					 ((src[c+1]>>16)&0xFF))/2;
		;
		sint32 b0=	 ((src[c+1]>> 8)&0xFF)
		;
		sint32 r1=	(((src[c+1]>>24)&0xFF)+
					 ((src[c+2]>>24)&0xFF))/2
		;
		sint32 g1=	 ((src[c+2]>>16)&0xFF)
		;
		sint32 b1=	(((src[c+2]>> 8)&0xFF)+
					 ((src[c+3]>> 8)&0xFF))
		;
		*dst ++ = (r0<<24) | (g0<<16) | b0;
		*dst ++ = (r1<<24) | (g1<<16) | b1;
	}
}

class ZSVGGraphic
{
	public:

		vector<ZPath> fillpaths;
		vector<ZPath> strokepaths;
		float64		 strokewidth;
		ZStencil	*fill;
		ZStencil	*stroke;
};

class MyIterator : public ZXMLIterator
{
	public:
	vector<ZSVGGraphic> graphic;
	ZOutliner outliner;

		void	ElementNamed(const ZXMLElement &element)
				{
				}

		void	ElementAttributed(const ZXMLElement &element)
				{
					if(strcmp(element.Name().c_str(),"g")==0)
					{
						ZSVGGraphic obj;
						obj.fill=0;
						obj.stroke=0;
						obj.strokewidth=-1;
						if(element.Attributes().size()>0)
						{
							for(sint32 d=0;d<element.Attributes().size();d++)
							{
//								OutputDebugString(element.Attributes()[d].c_str());
//								OutputDebugString("\n");
								if(strcmp(element.Attributes()[d].Name().c_str(),"style")==0)
								{
									sint32 c=0;
									for(;c<element.Attributes()[d].size();)
									{
										string str;
										for(;c<element.Attributes()[d].size();c++)
										{
											if(element.Attributes()[d][c]==';')
											{
												c++;
												break;
											}
											if(element.Attributes()[d][c]!=' ')
												str+=element.Attributes()[d][c];
										}

										if(strncmp(str.c_str(),"fill",4)==0)
										{
											sint32 f=str.find_first_of("#");
											uint32 color;
											sscanf(str.c_str()+f+1,"%06x",&color);
											obj.fill = new ZColorStencil(color|0xFF000000);
										}

										if(strncmp(str.c_str(),"stroke-width",12)==0)
										{
											sint32 f=str.find_first_of(":");
											float32 width=1;
											sscanf(str.c_str()+f+1,"%f",&width);
											obj.strokewidth = width;
										}

										if(strncmp(str.c_str(),"stroke",6)==0)
										{
											sint32 f=str.find_first_of("#");
											uint32 color;
											sscanf(str.c_str()+f+1,"%06x",&color);
											obj.stroke = new ZColorStencil(color|0xFF000000);
										}
									}
								}
							}
						}
						graphic.push_back(obj);
						return;
					}
					if(strcmp(element.Name().c_str(),"path")==0)
					{
						if(element.Attributes().size()>0)
						{
							for(sint32 d=0;d<element.Attributes().size();d++)
							{
								if(strcmp(element.Attributes()[d].Name().c_str(),"d")==0)
								{
									ZPath path;
									sint32 c=0;
									for(;c<element.Attributes()[d].size();)
									{
										string str;
										bool once=false;
										for(;c<element.Attributes()[d].size();c++)
										{
											if(once)
											{
												if(
													element.Attributes()[d][c]=='M' ||
													element.Attributes()[d][c]=='C' ||
													element.Attributes()[d][c]=='L'
												)
												{
													break;
												}
											}
											str+=element.Attributes()[d][c];
											once=true;
										}

										for(sint32 e=0;e<str.size();e++)
										{
											if(str[e]==',') str[e]=' ';
										}

										if(str[0]=='M')
										{
											float32 a=0; float32 b=0;
											sscanf(str.c_str()+1,"%f %f",&a,&b);
											path.AddMoveTo(ZPoint(a,b));
										}

										if(str[0]=='L')
										{
											float32 a=0; float32 b=0;
											sscanf(str.c_str()+1,"%f %f",&a,&b);
											path.AddLineTo(ZPoint(a,b));
										}
										if(str[0]=='C')
										{
											float32 a=ZMAXFLOAT32;
											float32 b=ZMAXFLOAT32;
											float32 c=ZMAXFLOAT32;
											float32 d=ZMAXFLOAT32;
											float32 e=ZMAXFLOAT32;
											float32 f=ZMAXFLOAT32;
											sscanf(str.c_str()+1,"%f %f %f %f %f %f",&a,&b,&c,&d,&e,&f);
											if(e==ZMAXFLOAT32)
											{
												path.AddCurveTo(ZPoint(a,b),ZPoint(c,d));
											}
											else
											{
												path.AddCurveTo(ZPoint(a,b),ZPoint(c,d),ZPoint(e,f));
											}
										}
									}
									if(graphic[graphic.size()-1].stroke)
									{
										if(graphic[graphic.size()-1].strokewidth==-1) graphic[graphic.size()-1].strokewidth=1;
										outliner.SetJointType(ZOutliner::JOINT_BEVEL);
										outliner.SetWidth(graphic[graphic.size()-1].strokewidth);
										graphic[graphic.size()-1].strokepaths.push_back(outliner.ComputeOutline(path));
									}
									if(graphic[graphic.size()-1].fill)
									{
										graphic[graphic.size()-1].fillpaths.push_back(path);
									}
								}
							}
						}
						return;
					}
				}
		void	ElementCompleted(const ZXMLElement &element)
				{
					if(strcmp(element.Name().c_str(),"g")==0)
					{
						return;
					}
				}
};

#ifdef WIN32
int APIENTRY WinMain(HINSTANCE hInstance,HINSTANCE hPrevInstance,LPSTR lpCmdLine,int nCmdShow)
#else
int main()
#endif
{
	MyIterator iter;
	ZXMLDocument doc;
	doc.Parse("tiger.svg",&iter);

	ZRenderer renderer;
	ZOutliner outliner;
	ZDasher dasher;

    try
    {
        // create console
        Console console;

        // create format
        Format format(32,0xFF0000,0x00FF00,0x0000FF);

        // open the console
        console.open("Zodius",320,240,format);

        // create surface matching console dimensions
        Surface surface(console.width(),console.height(),format);
		Surface surface_lcd(console.width()+console.width()/2,console.height(),format);

        ZMatrix matrix;
		float32 rot=0,scl=.4,scl1=0,rad=0,off=0,rdd=150;

		ZColorStencil stencil(ZColor(0xFFFF0000));
		ZColorStencil black(ZColor(0xFF000000));

//		ZLinearGradientStencil stencil;
//		stencil.SetColor(ZColor(0xFF,0xFF,0xFF,0x00),0,0.);
//		stencil.SetColor(ZColor(0xFF,0xFF,0x00,0x00),1,.5);
//		stencil.SetColor(ZColor(0xFF,0xFF,0x00,0xFF),2,1.);

//		ZRadialGradientStencil stencil;
//		ZLinearGradientStencil stencil;
//		stencil.SetColor(ZColor(255,255,255,255),0,0.);
//		stencil.SetColor(ZColor(255,255,  0,  0),1,.8);
//		stencil.SetColor(ZColor(255,200,200,200),2,1.);
//		stencil.SetVector(ZPoint(.5,.5),.5,ZPoint(.3,.3));

//		stencil.SetColor(ZColor(255,128,128,  0),2,1.);
/*		ZBitmapStencil stencil;
		uint32 *bitmap = new uint32[160*120];
		{FILE *fp = fopen("bitmap.raw","rb");
		if(fp)
		{
			sint32 c=0;
			for(;!feof(fp);)
			{
				if(c<160*120)
					bitmap[c++] = 0xFF000000 | (fgetc(fp)) | (fgetc(fp)<<8) | (fgetc(fp)<<16);
				else
					break;
			}
			ZColor::AlphaMultiply(bitmap,160*120);
			fclose(fp);
			int s=0;
		}}
		stencil.SetBitmap(bitmap,160,120);
*/
//		ZMatrix smtx;
//		smtx.SetScaling(ZPoint(.4,.4));
//		smtx.SetSkewing(ZPoint(.5,0));
//		stencil.SetStencilMatrix(smtx);

		sint32 cur=-1,cnt=0;
		ZColorStencil *stncs[64];
		ZPath *paths[64][64];

		memset(stncs,0,sizeof(stncs));
		memset(paths,0,sizeof(paths));

		{FILE *fp = fopen("lion.txt","r");
		if(fp)
		{
			char line[1024];
			for(;!feof(fp);)
			{
				fgets(line,1024,fp);
				if(line[0]!='M')
				{
					cur++;
					cnt=0;
					uint32 color = 0;
					sscanf(line,"%x",&color);
					color|=0x7F000000;
					stncs[cur] = new ZColorStencil(ZColor::AlphaMultiply(color));
				}
				else
				{
					paths[cur][cnt] = new ZPath();
					ZPath path;
					int32 x,y;
					char *scan = line;
					for(;sscanf(scan+1," %d,%d ",&x,&y);)
					{
						if(*scan=='M')
						{
							paths[cur][cnt]->AddMoveTo(ZPoint(x,y)-ZPoint(120,180));
						}
						else
						{
							paths[cur][cnt]->AddLineTo(ZPoint(x,y)-ZPoint(120,180));
						}
						if(*scan == 0) break;
						for(scan++;*scan!='L';scan++)
						{
							if(*scan == 0) break;
						}
					}
					paths[cur][cnt]->AddClose();
					cnt++;
				}
			}
			fclose(fp);
		}}

		static clock_t start = clock();
		static sint32  count = 0;

        // loop until a key is pressed
		for(;;)
		{
			count ++;

			Key key;
			if (console.key())
			{
				key = console.read();
			}

			// lock surface pixels
			uint32 *pixels = (uint32 *) surface_lcd.lock();

			// get surface dimensions
			const int width  = surface_lcd.width();
			const int height = surface_lcd.height();

			renderer.SetBuffer(pixels,width,height,width);
			renderer.ClearBuffer();

			ZPath path;
//			path.AddMoveTo(ZPoint( 10, 10));
//			path.AddLineTo(ZPoint(110, 10));
//			path.AddLineTo(ZPoint(110,110));
//			path.AddLineTo(ZPoint( 10,110));
//			path.AddClose();

			path.AddMoveTo(ZPoint(10,10)	- ZPoint(125,125));
			path.AddLineTo(ZPoint(200,10)	- ZPoint(125,125));
//			path.AddLineTo(ZPoint(200,200)	- ZPoint(125,125));
//			path.AddLineTo(ZPoint(10,200)	- ZPoint(125,125));
			path.AddArcTo(ZPoint(rdd,rdd+300),ZPoint(10,200)-ZPoint(125,125),1,1);
			path.AddClose();

/*			path.AddMoveTo(ZPoint(150,150)	- ZPoint(125,125));
			path.AddLineTo(ZPoint(250,150)	- ZPoint(125,125));
			path.AddLineTo(ZPoint(250,250)	- ZPoint(125,125));
			path.AddLineTo(ZPoint(150,250)	- ZPoint(125,125));
			path.AddClose();
*/
/*			path.AddMoveTo(ZPoint(20,20)	- ZPoint(125,125));
			path.AddCurveTo(ZPoint(150,20)- ZPoint(125,125),ZPoint(150,150)- ZPoint(125,125));
			path.AddCurveTo(ZPoint(20,150)- ZPoint(125,125),ZPoint(20,20)- ZPoint(125,125));
			path.AddClose();
*/
/*			path.AddMoveTo(ZPoint(-200,-200));
			path.AddCurveTo(ZPoint(200,-200),ZPoint(200,200));
			path.AddCurveTo(ZPoint(-200,200),ZPoint(200,-200));
			path.AddCurveTo(ZPoint(-300,200),ZPoint(-200,-200));
			path.AddClose();
*/
			switch (key.code())
            {
                case Key::O:	 off-=5.5f; break;
                case Key::P:	 off+=5.5f; break;
                case Key::C:	 scl-=.01f; break;
                case Key::V:	 scl+=.01f; break;
                case Key::Z:	 scl-=.25f; break;
                case Key::X:	 scl+=.25f; break;
                case Key::A:	 rad-=.25f; break;
                case Key::S:	 rad+=.25f; break;
                case Key::LEFT:  rot-=.01f; break;
                case Key::RIGHT: rot+=.01f; break;
                case Key::UP:    rot-=.10f; break;
                case Key::DOWN:  rot+=.10f; break;
				case Key::K:	 rdd+=5;	break;
				case Key::L:	 rdd-=5;	break;
            }

//			rot+=.10f;
//			scl+=.01f;
//			if(scl>1) scl=0;
//			scl1+=.013f;
//			if(scl1>1) scl1=0;

			ZDash dash;
			dash.SetLength(20,0);
			dash.SetLength(10,1);
			dash.SetLength(10,2);
			dash.SetLength(20,3);

#if 1
			outliner.SetJointType(ZOutliner::JOINT_NONE);
			outliner.SetWidth(32);
			renderer.SetSamplingSteps(16);
			dasher.SetOffset(off);

			for(float32 c=0;c<ZPI*2;c+=ZPI*2)
			{
				matrix.SetRotation(rot+c);
				matrix.SetScaling(ZPoint(scl,scl));
				matrix.SetTranslation(ZPoint(160,120));
				for(sint32 d=iter.graphic.size()-1;d>=0;d--)
				{
					if(iter.graphic[d].stroke)
					{
						for(sint32 e=0;e<iter.graphic[d].strokepaths.size();e++)
							renderer.DrawPath(iter.graphic[d].strokepaths[e],matrix,*iter.graphic[d].stroke);
					}
					if(iter.graphic[d].fill)
					{
						for(sint32 e=0;e<iter.graphic[d].fillpaths.size();e++)
							renderer.DrawPath(iter.graphic[d].fillpaths[e],matrix,*iter.graphic[d].fill);
					}
				}

//				renderer.DrawPath(outliner.ComputeOutline(path),matrix,stencil,ZRenderer::MODE_ZERO);
//				renderer.DrawPath(path,matrix,black,ZRenderer::MODE_ZERO);
			}
//			matrix.SetRotation(rot+1);
//			matrix.SetScaling(ZPoint(scl,scl));
//			matrix.SetTranslation(ZPoint(160,120));
//			renderer.DrawPath(path,matrix,stencil);
#else
			matrix.SetRotation(rot);
			matrix.SetScaling(ZPoint(scl,scl));
			matrix.SetTranslation(ZPoint(160,120));
			{for(sint32 c=0;c<64;c++)
			{
				for(sint32 i=0;i<64;i++)
				{
					if(paths[c][i])
					{
						renderer.DrawPath(*paths[c][i],matrix,*stncs[c]);
					}
				}
			}}
/*			matrix.SetScaling(ZPoint(scl1,scl1));
			matrix.SetTranslation(ZPoint(0+scl*250,80));
			{for(sint32 c=63;c>=0;c--)
			{
				for(sint32 i=63;i>=0;i--)
				{
					if(paths[c][i])
					{
						renderer.DrawPath(*paths[c][i],matrix,*stncs[c]);
					}
				}
			}}*/
#endif
			renderer.FinalizeBuffer(0xFFAACCAA);

			// unlock surface
			surface_lcd.unlock();

			uint32 *src = (uint32 *)surface_lcd.lock();
			uint32 *dst = (uint32 *)surface.lock();
			convert(src,dst);
			surface.unlock();
			surface_lcd.unlock();

			// copy to console
			surface.copy(console);

			// update console
			console.update();

			switch (key.code())
			{
				case Key::Q:
				case Key::ESCAPE: goto leave;
			}
		}

leave:

		start = clock()-start;
		float32 fps = (float32)(count) /
			(((float32)start)/CLOCKS_PER_SEC);

		char str[1024];
		sprintf(str,"%f frames/sec\n",fps);
#ifdef WIN32
		OutputDebugString(str);
#else
		printf("%s",str);
#endif

        // close console
        console.close();
    }
    catch (Error &error)
    {
        // report error
        error.report();
    }

    // exit
    return 0;
}
